@@ -150,17 +150,18 @@ from requests.auth import AuthBase
 from requests.packages import urllib3
 from requests.packages.urllib3.packages.ssl_match_hostname import match_hostname
 from threading import Lock
+from . import ssh_tunnel_app_support
 
 try:
+    # python-3
     from urllib.parse import urlsplit
 except ImportError:
+    # python-2
     from urlparse import urlsplit
 
-
 sequence_number_mutex = threading.Lock()
 counter = 0
 
-
 def _get_sequence_number():
     global counter
     with sequence_number_mutex:
@@ -251,16 +252,22 @@ def _get_env_var_proxy(print_proxy=False):
     if env_proxy in os.environ:
       proxy = os.environ[env_proxy]
   if print_proxy:
-    print('Using env variable %s=%s as proxy' % (env_proxy,proxy))
+    print('Using env variable %s=%s as proxy' % (env_proxy,proxy),
+          file=sys.stderr)
   return proxy
 
 def _get_pool_manager(verify, cert_file, key_file):
     global _pool_manager
     default_pool_args = dict(maxsize=32,
                              cert_reqs=ssl.CERT_REQUIRED,
-                             ca_certs=_default_certs,
                              headers=_default_headers,
                              timeout=_default_timeout)
+    # Don't use the default CA bundle if the user has set the env variable
+    # DX_USE_OS_CA_BUNDLE. Enabling that var will make us attempt to load
+    # the default CA certs provided by the OS; see DEVEX-875.
+    if 'DX_USE_OS_CA_BUNDLE' not in os.environ:
+        default_pool_args.update(ca_certs=_default_certs)
+
     if cert_file is None and verify is None and 'DX_CA_CERT' not in os.environ:
         with _pool_mutex:
             if _pool_manager is None:
@@ -333,6 +340,8 @@ def _is_retryable_exception(e):
         return True
     if isinstance(e, socket.error) and e.errno in _RETRYABLE_SOCKET_ERRORS:
         return True
+    if isinstance(e, urllib3.exceptions.NewConnectionError):
+        return True
     return False
 
 def _extract_msg_from_last_exception():
@@ -607,8 +616,32 @@ def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True,
                     return i
 
                 _headers = {ensure_ascii(k): ensure_ascii(v) for k, v in _headers.items()}
-                response = pool_manager.request(_method, _url, headers=_headers, body=body,
+                if USING_PYTHON2:
+                    encoded_url = _url
+                else:
+                    # This is needed for python 3 urllib
+                    _headers.pop(b'host', None)
+                    _headers.pop(b'content-length', None)
+                    _headers.pop(b'Content-Length', None)
+
+                    # The libraries downstream (http client) require elimination of non-ascii
+                    # chars from URL.
+                    # We check if the URL contains non-ascii characters to see if we need to
+                    # quote it. It is important not to always quote the path (here: parts[2])
+                    # since it might contain elements (e.g. HMAC for api proxy) containing
+                    # special characters that should not be quoted.
+                    try:
+                        ensure_ascii(_url)
+                        encoded_url = _url
+                    except UnicodeEncodeError:
+                        import urllib.parse
+                        parts = list(urllib.parse.urlparse(_url))
+                        parts[2] = urllib.parse.quote(parts[2])
+                        encoded_url = urllib.parse.urlunparse(parts)
+
+                response = pool_manager.request(_method, encoded_url, headers=_headers, body=body,
                                                 timeout=timeout, retries=False, **kwargs)
+
             except urllib3.exceptions.ClosedPoolError:
                 # If another thread closed the pool before the request was
                 # started, will throw ClosedPoolError
@@ -755,8 +788,8 @@ def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True,
                         waiting_msg = 'Waiting %d seconds before retry %d of %d...' % (
                             delay, try_index + 1, max_retries)
 
-                    logger.warn("[%s] %s %s: %s. %s %s",
-                                time.ctime(), method, _url, exception_msg, waiting_msg, range_str)
+                    logger.warning("[%s] %s %s: %s. %s %s",
+                                   time.ctime(), method, _url, exception_msg, waiting_msg, range_str)
                     time.sleep(delay)
                     try_index_including_503 += 1
                     if response is None or response.status != 503:
