@@ -385,7 +385,7 @@ def upload_resources(src_dir, project=None, folder='/', ensure_upload=False, for
                 # compress the file by reading the tar file and passing
                 # it though a GzipFile object, writing the given
                 # block size (by default 8192 bytes) at a time
-                targz_gzf = gzip.GzipFile(fileobj=targz_fh)
+                targz_gzf = gzip.GzipFile(fileobj=targz_fh, mode='wb')
                 tar_tmp_fh.seek(0)
                 dat = tar_tmp_fh.read(io.DEFAULT_BUFFER_SIZE)
                 while dat:
@@ -566,6 +566,7 @@ def upload_applet(src_dir, uploaded_resources, check_name_collisions=True, overw
     for asset in asset_depends:
         asset_project = asset.get("project", None)
         asset_folder = asset.get("folder", '/')
+        asset_stages = asset.get("stages", None)
         if "id" in asset:
             asset_record = dxpy.DXRecord(asset["id"]).describe(fields={'details'}, default_fields=True)
         elif "name" in asset and asset_project is not None and "version" in asset:
@@ -597,6 +598,8 @@ def upload_applet(src_dir, uploaded_resources, check_name_collisions=True, overw
                 "name": archive_file_name,
                 "id": archive_file_id
             }
+            if asset_stages:
+                bundle_depends["stages"] = asset_stages
             applet_spec["runSpec"]["bundledDepends"].append(bundle_depends)
             # If the file is not found in the applet destination project, clone it from the asset project
             if (not dry_run and
@@ -709,7 +712,7 @@ def create_app_multi_region(regional_options, app_name, src_dir, publish=False,
     Creates a new app object from the specified applet(s).
 
     :param regional_options: Region-specific options for the app. See
-        https://wiki.dnanexus.com/API-Specification-v1.0.0/Apps#API-method:-/app/new
+        https://documentation.dnanexus.com/developer/api/running-analyses/apps#api-method-app-new
         for details; this should contain keys for each region the app is
         to be enabled in, and for the values, a dict containing (at
         minimum) a key "applet" whose value is an applet ID for that
@@ -912,20 +915,6 @@ def _create_app(applet_or_regional_options, app_name, src_dir, publish=False, se
     return app_id
 
 
-def assert_consistent_regions(from_app_spec, from_command_line):
-    """
-    :param from_app_spec: The regional options specified in dxapp.json.
-    :type from_app_spec: dict or None.
-    :param from_command_line: The regional options specified on the
-    command-line via --region.
-    :type from_command_line: list or None.
-    """
-    if from_app_spec is None or from_command_line is None:
-        return
-    if set(from_app_spec) != set(from_command_line):
-        raise dxpy.app_builder.AppBuilderException("--region and the 'regionalOptions' key in dxapp.json do not agree")
-
-
 def get_enabled_regions(app_spec, from_command_line):
     """Returns a list of the regions in which the app should be enabled.
 
@@ -939,42 +928,7 @@ def get_enabled_regions(app_spec, from_command_line):
     :type from_command_line: list or None
 
     """
-    from_app_spec = app_spec.get('regionalOptions')
-
-    if from_app_spec is not None:
-        if not isinstance(from_app_spec, dict):
-            raise dxpy.app_builder.AppBuilderException("The field 'regionalOptions' in dxapp.json must be a mapping")
-        if not from_app_spec:
-            raise dxpy.app_builder.AppBuilderException(
-                "The field 'regionalOptions' in dxapp.json must be a non-empty mapping")
-        regional_options_list = list(from_app_spec.items())
-        for region, opts_for_region in regional_options_list:
-            if not isinstance(opts_for_region, dict):
-                raise dxpy.app_builder.AppBuilderException("The field 'regionalOptions['" + region +
-                                                           "']' in dxapp.json must be a mapping")
-            if set(opts_for_region.keys()) != set(regional_options_list[0][1].keys()):
-                if set(opts_for_region.keys()) - set(regional_options_list[0][1].keys()):
-                    with_key, without_key = region, regional_options_list[0][0]
-                    key_name = next(iter(set(opts_for_region.keys()) - set(regional_options_list[0][1].keys())))
-                else:
-                    with_key, without_key = regional_options_list[0][0], region
-                    key_name = next(iter(set(regional_options_list[0][1].keys()) - set(opts_for_region.keys())))
-                raise dxpy.app_builder.AppBuilderException(
-                    "All regions in regionalOptions must specify the same options; " +
-                    "%s was given for %s but not for %s" % (key_name, with_key, without_key)
-                )
-            for key in opts_for_region:
-                if key in app_spec.get('runSpec', {}):
-                    raise dxpy.app_builder.AppBuilderException(
-                        key + " cannot be given in both runSpec and in regional options for " + region)
-
-    assert_consistent_regions(from_app_spec, from_command_line)
-
-    enabled_regions = None
-    if from_app_spec is not None:
-        enabled_regions = from_app_spec.keys()
-    elif from_command_line is not None:
-        enabled_regions = from_command_line
+    enabled_regions = dxpy.executable_builder.get_enabled_regions('app', app_spec, from_command_line, AppBuilderException)
 
     if enabled_regions is not None and len(enabled_regions) == 0:
         raise AssertionError("This app should be enabled in at least one region")
