@@ -25,7 +25,7 @@ from __future__ import print_function, unicode_literals, division, absolute_impo
 import time, copy, re
 
 import dxpy.api
-from ..exceptions import (DXError, DXAPIError, DXFileError, DXGTableError, DXSearchError, DXAppletError,
+from ..exceptions import (DXError, DXAPIError, DXFileError, DXSearchError, DXAppletError,
                           DXJobFailureError, AppError, AppInternalError, DXCLIError)
 from ..compat import basestring
 
@@ -36,6 +36,7 @@ def verify_string_dxid(dxid, expected_classes):
     :type expected_classes: string or list of strings
     :raises: :exc:`~dxpy.exceptions.DXError` if *dxid* is not a string or is not a valid DNAnexus ID of the expected class
     '''
+
     if isinstance(expected_classes, basestring):
         expected_classes = [expected_classes]
     if not isinstance(expected_classes, list) or len(expected_classes) == 0:
@@ -66,12 +67,12 @@ class DXObject(object):
         dxproj_id = self._proj if self._proj is not None else "no project ID stored"
 
         if use_name:
-            if self._class not in ["container", "project", "app"]:
+            if self._class not in ["container", "project", "app", "globalworkflow"]:
                 desc = "<dxpy.{classname}: {name} ({dxid} ({dxproj_id}))>"
             else:
                 desc = "<dxpy.{classname}: {name} ({dxid})>"
         else:
-            if self._class not in ["container", "project", "app"]:
+            if self._class not in ["container", "project", "app", "globalworkflow"]:
                 desc = "<{module}.{classname} object at 0x{mem_loc:x}: {dxid} ({dxproj_id})>"
             else:
                 desc = "<{module}.{classname} object at 0x{mem_loc:x}: {dxid}>"
@@ -91,7 +92,13 @@ class DXObject(object):
         return self._repr()
 
     def __getattr__(self, attr):
+        if attr == "__setstate__":
+            # without this, an infinite recursion occurs when copying an object.
+            # https://stackoverflow.com/questions/47299243/recursionerror-when-python-copy-deepcopy
+            raise AttributeError(attr)
         if not self._desc:
+            # describe can fail if the object has not been created
+            # properly. For example, it has an invalid dxid.
             self.describe()
         try:
             return self._desc[attr]
@@ -275,6 +282,7 @@ class DXDataObject(DXObject):
         data container is used).
 
         '''
+
         if is_dxlink(dxid):
             dxid, project_from_link = get_dxlink_ids(dxid)
             if project is None:
@@ -633,6 +641,7 @@ class DXDataObject(DXObject):
 
     def _wait_on_close(self, timeout=3600*24*1, **kwargs):
         elapsed = 0
+        i = 0
         while True:
             state = self._get_state(**kwargs)
             if state == "closed":
@@ -643,14 +652,16 @@ class DXDataObject(DXObject):
             if elapsed >= timeout or elapsed < 0:
                 raise DXError("Reached timeout while waiting for the remote object to close")
 
-            time.sleep(2)
-            elapsed += 2
+            wait = min(2**7, 2**i)
+            time.sleep(wait)
+            i += 1
+            elapsed += wait
 
 from .dxfile import DXFile, DXFILE_HTTP_THREADS, DEFAULT_BUFFER_SIZE
+from .dxdatabase import DXDatabase, DXFILE_HTTP_THREADS, DEFAULT_BUFFER_SIZE
 from .download_all_inputs import download_all_inputs
 from .dxfile_functions import open_dxfile, new_dxfile, download_dxfile, upload_local_file, upload_string, list_subfolders, download_folder
-from .dxgtable import DXGTable, NULL, DXGTABLE_HTTP_THREADS
-from .dxgtable_functions import open_dxgtable, new_dxgtable
+from .dxdatabase_functions import download_dxdatabasefile
 from .dxrecord import DXRecord, new_dxrecord
 from .dxproject import DXContainer, DXProject
 from .dxjob import DXJob, new_dxjob
