@@ -29,17 +29,18 @@ import hashlib
 import traceback
 import warnings
 from collections import defaultdict
+import multiprocessing
 
 import dxpy
 from .. import logger
 from . import dxfile, DXFile
 from .dxfile import FILE_REQUEST_TIMEOUT
-from ..compat import open
+from ..compat import open, USING_PYTHON2
 from ..exceptions import DXFileError, DXPartLengthMismatchError, DXChecksumMismatchError, DXIncompleteReadsError, err_exit
 from ..utils import response_iterator
 import subprocess
 
-def open_dxfile(dxid, project=None, read_buffer_size=dxfile.DEFAULT_BUFFER_SIZE):
+def open_dxfile(dxid, project=None, mode=None, read_buffer_size=dxfile.DEFAULT_BUFFER_SIZE):
     '''
     :param dxid: file ID
     :type dxid: string
@@ -59,7 +60,7 @@ def open_dxfile(dxid, project=None, read_buffer_size=dxfile.DEFAULT_BUFFER_SIZE)
       DXFile(dxid)
 
     '''
-    return DXFile(dxid, project=project, read_buffer_size=read_buffer_size)
+    return DXFile(dxid, project=project, mode=mode, read_buffer_size=read_buffer_size)
 
 
 def new_dxfile(mode=None, write_buffer_size=dxfile.DEFAULT_BUFFER_SIZE, expected_file_size=None, file_is_mmapd=False,
@@ -94,7 +95,7 @@ def new_dxfile(mode=None, write_buffer_size=dxfile.DEFAULT_BUFFER_SIZE, expected
 
 
 def download_dxfile(dxid, filename, chunksize=dxfile.DEFAULT_BUFFER_SIZE, append=False, show_progress=False,
-                    project=None, **kwargs):
+                    project=None, describe_output=None, **kwargs):
     '''
     :param dxid: DNAnexus file ID or DXFile (file handler) object
     :type dxid: string or DXFile
@@ -106,7 +107,11 @@ def download_dxfile(dxid, filename, chunksize=dxfile.DEFAULT_BUFFER_SIZE, append
             which billing account is billed for this download). If None or
             DXFile.NO_PROJECT_HINT, no project hint is supplied to the API server.
     :type project: str or None
-
+    :param describe_output: (experimental) output of the file-xxxx/describe API call,
+            if available. It will make it possible to skip another describe API call.
+            It should contain the default fields of the describe API call output and
+            the "parts" field, not included in the output by default.
+    :type describe_output: dict or None
 
     Downloads the remote file referenced by *dxid* and saves it to *filename*.
 
@@ -119,9 +124,15 @@ def download_dxfile(dxid, filename, chunksize=dxfile.DEFAULT_BUFFER_SIZE, append
     part_retry_counter = defaultdict(lambda: 3)
     success = False
     while not success:
-        success = _download_dxfile(dxid, filename, part_retry_counter,
-                                   chunksize=chunksize, append=append,
-                                   show_progress=show_progress, project=project, **kwargs)
+        success = _download_dxfile(dxid,
+                                   filename,
+                                   part_retry_counter,
+                                   chunksize=chunksize,
+                                   append=append,
+                                   show_progress=show_progress,
+                                   project=project,
+                                   describe_output=describe_output,
+                                   **kwargs)
 
 
 # Check if a program (wget, curl, etc.) is on the path, and
@@ -153,8 +164,11 @@ def _verify(filename, md5digest):
     if len(line) != 2:
         err_exit("md5sum returned weird results: " + str(line))
     actual_md5 = line[0]
+    md5digest = md5digest.encode("ascii")
+
+    # python-3 : both digests have to be in bytes
     if actual_md5 != md5digest:
-        err_exit("Checksum doesn't match " + actual_md5 + "  expected:" + md5digest)
+        err_exit("Checksum doesn't match " + str(actual_md5) + "  expected:" + str(md5digest))
     print("Checksum correct")
 
 # [dxid] is a symbolic link. Create a preauthenticated URL,
@@ -164,35 +178,53 @@ def _download_symbolic_link(dxid, md5digest, project, dest_filename):
     url, _headers = dxfile.get_download_url(preauthenticated=True,
                                             duration=6*3600,
                                             project=project)
- 
+
     # Follow the redirection
     print('Following redirect for ' + url)
 
-    wget_exe = _which("wget")
-    if wget_exe is None:
-        err_exit("wget is not installed on this system")
+    # Check if aria2 present
+    # Use that instead of wget
+    aria2c_exe = _which("aria2c")
+    if aria2c_exe is None:
+        wget_exe = _which("wget")
+        if wget_exe is None:
+            err_exit("wget is not installed on this system")
 
-    cmd = ["wget", "--tries=5", "--quiet"]
-    if os.path.isfile(dxid):
-        # file already exists, resume upload.
-        cmd += ["--continue"]
-    cmd += ["-O", dest_filename, url]
+        cmd = ["wget", "--tries=5", "--quiet"]
+        if os.path.isfile(dxid):
+            # file already exists, resume upload.
+            cmd += ["--continue"]
+        cmd += ["-O", dest_filename, url]
+    else:
+        print("aria2c found in path so using that instead of wget \n")
+        # aria2c does not allow more than 16 connections per server
+        max_connections = min(16, multiprocessing.cpu_count())
+        cmd = ["aria2c", "--check-certificate=false", "-s", str(max_connections), "-x", str(max_connections), "--retry-wait=10", "--max-tries=15"]
+        # Split path properly for aria2c
+        # If '-d' arg not provided, aria2c uses current working directory
+        cwd = os.getcwd()
+        directory, filename = os.path.split(dest_filename)
+        directory = cwd if directory in ["", cwd] else directory
+        cmd += ["-o", filename, "-d", os.path.abspath(directory), url]
 
     try:
-        print("Downloading symbolic link with wget")
+        if aria2c_exe is not None:
+            print("Downloading symbolic link with aria2c")
+        else:
+            print("Downloading symbolic link with wget")
         subprocess.check_call(cmd, stderr=subprocess.STDOUT)
     except subprocess.CalledProcessError as e:
         msg = ""
         if e and e.output:
             msg = e.output.strip()
-        err_exit("Failed to call wget: {cmd}\n{msg}\n".format(cmd=str(cmd), msg=msg))
+        err_exit("Failed to call download: {cmd}\n{msg}\n".format(cmd=str(cmd), msg=msg))
 
     if md5digest is not None:
         _verify(dest_filename, md5digest)
 
 def _download_dxfile(dxid, filename, part_retry_counter,
                      chunksize=dxfile.DEFAULT_BUFFER_SIZE, append=False, show_progress=False,
-                     project=None, **kwargs):
+                     project=None, describe_output=None, **kwargs):
     '''
     Core of download logic. Download file-id *dxid* and store it in
     a local file *filename*.
@@ -234,9 +266,11 @@ def _download_dxfile(dxid, filename, part_retry_counter,
     else:
         dxfile = DXFile(dxid, mode="r", project=(project if project != DXFile.NO_PROJECT_HINT else None))
 
-    dxfile_desc = dxfile.describe(fields={"parts"}, default_fields=True, **kwargs)
+    if describe_output and describe_output.get("parts") is not None:
+        dxfile_desc = describe_output
+    else:
+        dxfile_desc = dxfile.describe(fields={"parts"}, default_fields=True, **kwargs)
 
-    from pprint import pprint
     if 'drive' in dxfile_desc:
         # A symbolic link. Get the MD5 checksum, if we have it
         if 'md5' in dxfile_desc:
@@ -553,7 +587,6 @@ def upload_string(to_upload, media_type=None, keep_open=False, wait_on_close=Fal
     # For subsequent API calls, don't supply the dataobject metadata
     # parameters that are only needed at creation time.
     _, remaining_kwargs = dxpy.DXDataObject._get_creation_params(kwargs)
-
     handler.write(to_upload, **remaining_kwargs)
 
     if not keep_open:
@@ -635,9 +668,25 @@ def download_folder(project, destdir, folder="/", overwrite=False, chunksize=dxf
         ensure_local_dir(compose_local_dir(normalized_dest_dir, normalized_folder, remote_subfolder))
 
     # Downloading files
-    describe_input = dict(fields=dict(folder=True, name=True, id=True))
-    for remote_file in dxpy.search.find_data_objects(classname='file', state='closed', project=project,
-                                                     folder=normalized_folder, recurse=True, describe=describe_input):
+    describe_input = dict(fields=dict(folder=True,
+                                      name=True,
+                                      id=True,
+                                      parts=True,
+                                      size=True,
+                                      drive=True,
+                                      md5=True))
+
+    # A generator that returns the files one by one. We don't want to materialize it, because
+    # there could be many files here.
+    files_gen = dxpy.search.find_data_objects(classname='file', state='closed', project=project,
+                                              folder=normalized_folder, recurse=True, describe=describe_input)
+    if files_gen is None:
+        # In python 3, the generator can be None, and iterating on it
+        # will cause an error.
+        return
+
+    # Now it is safe, in both python 2 and 3, to iterate on the generator
+    for remote_file in files_gen:
         local_filename = os.path.join(compose_local_dir(normalized_dest_dir,
                                                         normalized_folder,
                                                         remote_file['describe']['folder']),
@@ -650,5 +699,10 @@ def download_folder(project, destdir, folder="/", overwrite=False, chunksize=dxf
                      ("" if remote_file['describe']['folder'] == "/" else remote_file['describe']['folder']),
                      remote_file['describe']['name'],
                      local_filename)
-        download_dxfile(remote_file['describe']['id'], local_filename, chunksize=chunksize, project=project,
-                        show_progress=show_progress, **kwargs)
+        download_dxfile(remote_file['describe']['id'],
+                        local_filename,
+                        chunksize=chunksize,
+                        project=project,
+                        show_progress=show_progress,
+                        describe_output=remote_file['describe'],
+                        **kwargs)
