@@ -16,6 +16,7 @@
 #   License for the specific language governing permissions and limitations
 #   under the License.
 
+
 from __future__ import print_function, unicode_literals, division, absolute_import
 
 import logging
@@ -107,7 +108,7 @@ def _check_suggestions(app_json, publish=False):
                         logger.warn('Folder {path} could not be found in project {project}'.format(
                                      path=suggestion['path'], project=suggestion['project']))
             if '$dnanexus_link' in suggestion:
-                if suggestion['$dnanexus_link'].startswith(('file-', 'record-', 'gtable-')):
+                if suggestion['$dnanexus_link'].startswith(('file-', 'record-')):
                     try:
                         dnanexus_link = dxpy.describe(suggestion['$dnanexus_link'])
                     except dxpy.exceptions.DXAPIError as e:
@@ -117,7 +118,7 @@ def _check_suggestions(app_json, publish=False):
                                     name=suggestion['$dnanexus_link']))
                     except Exception as e:
                         raise dxpy.app_builder.AppBuilderException(str(e))
-            if 'value' in suggestion:
+            if 'value' in suggestion and isinstance(suggestion["value"], dict):
                 if '$dnanexus_link' in suggestion['value']:
                     # Check if we have JSON or string
                     if isinstance(suggestion['value']['$dnanexus_link'], dict):
@@ -129,7 +130,7 @@ def _check_suggestions(app_json, publish=False):
                                     logger.warn('Suggested project {name} does not exist, or not accessible by user'.format(
                                                  name=suggestion['value']['$dnanexus_link']['project']))
                     elif isinstance(suggestion['value']['$dnanexus_link'], basestring):
-                        if suggestion['value']['$dnanexus_link'].startswith(('file-', 'record-', 'gtable-')):
+                        if suggestion['value']['$dnanexus_link'].startswith(('file-', 'record-')):
                             try:
                                 dnanexus_link = dxpy.describe(suggestion['value']['$dnanexus_link'])
                             except dxpy.exceptions.DXAPIError as e:
@@ -293,13 +294,21 @@ def _check_file_syntax(filename, temp_dir, override_lang=None, enforce=True):
         checker_fn(filename)
     except subprocess.CalledProcessError as e:
         print(filename + " has a syntax error! Interpreter output:", file=sys.stderr)
-        for line in e.output.strip("\n").split("\n"):
+        if USING_PYTHON2:
+            errmsg = e.output
+        else:
+            errmsg = e.output.decode("utf-8")
+        for line in errmsg.strip("\n").split("\n"):
             print("  " + line.rstrip("\n"), file=sys.stderr)
         if enforce:
             raise DXSyntaxError(filename + " has a syntax error")
     except py_compile.PyCompileError as e:
         print(filename + " has a syntax error! Interpreter output:", file=sys.stderr)
-        print("  " + e.msg.strip(), file=sys.stderr)
+        try:
+            errmsg = e.msg.decode("utf-8")
+        except AttributeError:
+            errmsg = e.msg
+        print("  " + errmsg.strip(), file=sys.stderr)
         if enforce:
             raise DXSyntaxError(e.msg.strip())
 
@@ -363,7 +372,7 @@ def _verify_app_source_dir_impl(src_dir, temp_dir, mode, enforce=True):
                 if not isinstance(manifest.get('access'), dict) or 'network' not in manifest['access']:
                     msg = '\n'.join(['runSpec.execDepends specifies non-APT dependencies, but no network access spec is given.',
                     'Add {"access": {"network": ["*"]}} to allow dependencies to install.',
-                    'See https://wiki.dnanexus.com/Developer-Tutorials/Request-Additional-App-Resources#Network-Access.',
+                    'See https://documentation.dnanexus.com/developer/apps/execution-environment#network-access.',
                     'Rerun with --no-check-syntax to proceed anyway.'])
                     raise dxpy.app_builder.AppBuilderException(msg)
 
@@ -464,7 +473,7 @@ def _parse_app_spec(src_dir):
         try:
             return json_load_raise_on_duplicates(app_desc)
         except Exception as e:
-            raise dxpy.app_builder.AppBuilderException("Could not parse dxapp.json file as JSON: " + e.message)
+            raise dxpy.app_builder.AppBuilderException("Could not parse dxapp.json file as JSON: " + str(e.args))
 
 def _build_app_remote(mode, src_dir, publish=False, destination_override=None,
                       version_override=None, bill_to_override=None, dx_toolkit_autodep="stable",
@@ -476,8 +485,9 @@ def _build_app_remote(mode, src_dir, publish=False, destination_override=None,
         builder_app = 'app-tarball_applet_builder'
 
     app_spec = _parse_app_spec(src_dir)
-    if app_spec['runSpec'].get('release') == '14.04':
-        builder_app += "_trusty"
+    builder_versions = {"12.04": "", "14.04": "_trusty", "16.04": "_xenial",
+                        "18.04": "_bionic", "20.04": "_focal"}
+    builder_app += builder_versions.get(app_spec['runSpec'].get('release'), "")
 
     temp_dir = tempfile.mkdtemp()
 
@@ -660,7 +670,6 @@ def build_and_upload_locally(src_dir, mode, overwrite=False, archive=False, publ
                              dx_toolkit_autodep="stable", do_check_syntax=True, dry_run=False,
                              return_object_dump=False, confirm=True, ensure_upload=False, force_symlinks=False,
                              region=None, **kwargs):
-
     dxpy.app_builder.build(src_dir, parallel_build=do_parallel_build)
     app_json = _parse_app_spec(src_dir)
     _check_suggestions(app_json, publish=publish)
@@ -704,7 +713,7 @@ def build_and_upload_locally(src_dir, mode, overwrite=False, archive=False, publ
             except:
                 # A /project/new request may fail if the requesting user is
                 # not authorized to create projects in a certain region.
-                dxpy.executable_builder.delete_temporary_projects(projects_by_region.values())
+                dxpy.executable_builder.delete_temporary_projects(list(projects_by_region.values()))
                 err_exit()
         else:
             # Create a temp project
@@ -785,7 +794,7 @@ def build_and_upload_locally(src_dir, mode, overwrite=False, archive=False, publ
             raise dxpy.app_builder.AppBuilderException(error_message)
 
         resources_bundles_by_region = {}
-        for region, project in projects_by_region.items():
+        for region, project in list(projects_by_region.items()):
             resources_bundles_by_region[region] = dxpy.app_builder.upload_resources(
                 src_dir,
                 project=project,
@@ -796,7 +805,7 @@ def build_and_upload_locally(src_dir, mode, overwrite=False, archive=False, publ
         # TODO: Clean up these applets if the app build fails.
         applet_ids_by_region = {}
         try:
-            for region, project in projects_by_region.items():
+            for region, project in list(projects_by_region.items()):
                 applet_id, applet_spec = dxpy.app_builder.upload_applet(
                     src_dir,
                     resources_bundles_by_region[region],
@@ -818,7 +827,7 @@ def build_and_upload_locally(src_dir, mode, overwrite=False, archive=False, publ
             # using_temp_project, the entire project gets destroyed at
             # the end, so we don't bother.
             if not using_temp_project:
-                for region, project in projects_by_region.items():
+                for region, project in list(projects_by_region.items()):
                     objects_to_delete = [dxpy.get_dxlink_ids(bundled_resource_obj['id'])[0] for bundled_resource_obj in resources_bundles_by_region[region]]
                     if objects_to_delete:
                         dxpy.api.project_remove_objects(
@@ -841,11 +850,11 @@ def build_and_upload_locally(src_dir, mode, overwrite=False, archive=False, publ
 
             additional_resources_by_region = {}
             if "regionalOptions" in app_json:
-                for region, region_config in app_json["regionalOptions"].items():
+                for region, region_config in list(app_json["regionalOptions"].items()):
                     if "resources" in region_config:
                         additional_resources_by_region[region] = region_config["resources"]
             elif "resources" in app_json:
-                additional_resources_by_region[projects_by_region.keys()[0]] = app_json["resources"]
+                additional_resources_by_region[list(projects_by_region.keys())[0]] = app_json["resources"]
 
             regional_options = {}
             for region in projects_by_region:
@@ -855,7 +864,7 @@ def build_and_upload_locally(src_dir, mode, overwrite=False, archive=False, publ
 
             # add pricingPolicy separately for better readability
             if "regionalOptions" in app_json:
-                for region, region_config in app_json["regionalOptions"].items():
+                for region, region_config in list(app_json["regionalOptions"].items()):
                     if "pricingPolicy" in region_config:
                         regional_options[region]["pricingPolicy"] = region_config["pricingPolicy"]
 
@@ -889,7 +898,7 @@ def build_and_upload_locally(src_dir, mode, overwrite=False, archive=False, publ
     finally:
         # Clean up after ourselves.
         if using_temp_project:
-            dxpy.executable_builder.delete_temporary_projects(projects_by_region.values())
+            dxpy.executable_builder.delete_temporary_projects(list(projects_by_region.values()))
 
 
 def _build_app(args, extra_args):
@@ -933,7 +942,7 @@ def _build_app(args, extra_args):
         except dxpy.app_builder.AppBuilderException as e:
             # AppBuilderException represents errors during app or applet building
             # that could reasonably have been anticipated by the user.
-            print("Error: %s" % (e.message,), file=sys.stderr)
+            print("Error: %s" % (e.args,), file=sys.stderr)
             sys.exit(3)
         except dxpy.exceptions.DXAPIError as e:
             print("Error: %s" % (e,), file=sys.stderr)
@@ -954,7 +963,7 @@ def _build_app(args, extra_args):
             if args.mode == "app" and not args.dry_run:
                 dxpy.executable_builder.verify_developer_rights('app-' + app_json['name'])
         except dxpy.app_builder.AppBuilderException as e:
-            print("Error: %s" % (e.message,), file=sys.stderr)
+            print("Error: %s" % (e.args,), file=sys.stderr)
             sys.exit(3)
 
         # The following flags might be useful in conjunction with
