@@ -337,7 +337,7 @@ def create_files_from_templates(template_dir, app_json, language,
 
 This is the source code for an app that runs on the DNAnexus Platform.
 For more information about how to run or modify it, see
-https://wiki.dnanexus.com/.
+https://documentation.dnanexus.com/.
 <!-- /dx-header -->
 
 {description}
@@ -349,7 +349,7 @@ your app that would be helpful to users. (Also see the
 Readme.developer.md.) Once you're done, you can remove these TODO
 comments.
 
-For more info, see https://wiki.dnanexus.com/Developer-Portal.
+For more info, see https://documentation.dnanexus.com/developer.
 -->
 '''
     with open(os.path.join(name, 'Readme.md'), 'w') as readme_file:
@@ -384,7 +384,7 @@ This app has the following entry points:
     }}
 
 See <a
-href="https://wiki.dnanexus.com/API-Specification-v1.0.0/IO-and-Run-Specifications#Run-Specification">Run
+href="https://documentation.dnanexus.com/developer/api/running-analyses/io-and-run-specifications#run-specification">Run
 Specification</a> in the API documentation for more information about the
 available instance types.
 '''
