@@ -53,6 +53,7 @@ def run(command, **kwargs):
 TEST_APPS = os.path.join(os.path.dirname(__file__), 'file_load')
 LOCAL_SCRIPTS = os.path.join(os.path.dirname(__file__), '..', 'scripts')
 LOCAL_UTILS = os.path.join(os.path.dirname(__file__), '..', 'dxpy', 'utils')
+DUMMY_HASH = "123456789012345678901234"
 
 
 def ignore_folders(directory, contents):
@@ -73,7 +74,7 @@ def build_app_with_bash_helpers(app_dir, project_id):
         shutil.copytree(app_dir, updated_app_dir)
         # Copy the current verion of dx-toolkit. We will build it on the worker
         # and source this version which will overload the stock version of dx-toolkit.
-        # This we we can test all bash helpers as they would appear locally with all
+        # This way we can test all bash helpers as they would appear locally with all
         # necessary dependencies
         #dxtoolkit_dir = os.path.abspath(os.path.join(updated_app_dir, 'resources', 'dxtoolkit'))
         #local_dxtoolkit = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..', '..'))
@@ -87,7 +88,7 @@ def build_app_with_bash_helpers(app_dir, project_id):
         preamble.append('sudo pip install --upgrade virtualenv\n')
         #preamble.append('make -C {toolkitdir} python\n'.format(toolkitdir=dxtoolkit_dir))
         #preamble.append('source {toolkitdir}/environment\n'.format(toolkitdir=dxtoolkit_dir))
-        preamble.append('make -C /dxtoolkit python\n')
+        preamble.append('make -C /dxtoolkit clean python\n')
         preamble.append('source /dxtoolkit/environment\n')
         # Now find the applet entry point file and prepend the
         # operations above, overwriting it in place.
@@ -251,6 +252,25 @@ class TestDXBashHelpers(DXTestCase):
             cmd_args.extend(applet_args)
             run(cmd_args, env=env)
 
+    def test_xattr_parameters(self):
+        ''' Tests dx-upload-all-outputs uploading with filesystem metadata as properties '''
+        with temporary_project('TestDXBashHelpers.test_app1 temporary project') as dxproj:
+            env = update_environ(DX_PROJECT_CONTEXT_ID=dxproj.get_id())
+
+            # Upload some files for use by the applet
+            dxpy.upload_string("1234\n", project=dxproj.get_id(), name="A.txt")
+            dxpy.upload_string("ABCD\n", project=dxproj.get_id(), name="B.txt")
+
+            # Build the applet, patching in the bash helpers from the
+            # local checkout
+            applet_id = build_app_with_bash_helpers(os.path.join(TEST_APPS, 'xattr_properties'), dxproj.get_id())
+
+            # Run the applet
+            applet_args = ["-iseq1=A.txt", "-iseq2=B.txt", "-iref=A.txt", "-iref=B.txt"]
+            cmd_args = ['dx', 'run', '--yes', '--watch', applet_id]
+            cmd_args.extend(applet_args)
+            run(cmd_args, env=env)
+
     @unittest.skipUnless(testutil.TEST_RUN_JOBS, 'skipping test that would run a job')
     def test_file_optional(self):
         ''' Tests that optional and non-optional file output arguments are
@@ -418,12 +438,10 @@ class TestDXBashHelpersBenchmark(DXTestCase):
     def test_par_1g(self):
         self.run_applet_with_flags(["-iparallel=true"], 10, 1024 * 1024 * 1024)
 
-
 class TestDXJobutilAddOutput(DXTestCase):
-    dummy_hash = "123456789012345678901234"
     data_obj_classes = ['file', 'record', 'applet', 'workflow']
-    dummy_ids = [obj_class + '-' + dummy_hash for obj_class in data_obj_classes]
-    dummy_job_id = "job-" + dummy_hash
+    dummy_ids = [ (obj_class + '-' + DUMMY_HASH) for obj_class in data_obj_classes]
+    dummy_job_id = "job-" + DUMMY_HASH
     dummy_analysis_id = "analysis-123456789012345678901234"
     test_cases = ([["32", 32],
                    ["3.4", 3.4],
@@ -440,7 +458,7 @@ class TestDXJobutilAddOutput(DXTestCase):
                     {"$dnanexus_link": dummy_id}] for dummy_id in dummy_ids])
 
     def test_auto(self):
-        with tempfile.NamedTemporaryFile() as f:
+        with tempfile.NamedTemporaryFile(mode='w+') as f:
             # initialize the file with valid JSON
             f.write('{}')
             f.flush()
@@ -454,7 +472,7 @@ class TestDXJobutilAddOutput(DXTestCase):
                 self.assertEqual(result[str(i)], tc[1])
 
     def test_auto_array(self):
-        with tempfile.NamedTemporaryFile() as f:
+        with tempfile.NamedTemporaryFile(mode='w+') as f:
             # initialize the file with valid JSON
             f.write('{}')
             f.flush()
@@ -469,7 +487,7 @@ class TestDXJobutilAddOutput(DXTestCase):
                 self.assertEqual(result[str(i)], [tc[1], tc[1]])
 
     def test_class_specific(self):
-        with tempfile.NamedTemporaryFile() as f:
+        with tempfile.NamedTemporaryFile(mode='w+') as f:
             # initialize the file with valid JSON
             f.write('{}')
             f.flush()
@@ -486,7 +504,7 @@ class TestDXJobutilAddOutput(DXTestCase):
                 self.assertEqual(result[str(i)], tc[2])
 
     def test_class_parsing_errors(self):
-        with tempfile.NamedTemporaryFile() as f:
+        with tempfile.NamedTemporaryFile(mode='w+') as f:
             # initialize the file with valid JSON
             f.write('{}')
             f.flush()
@@ -501,30 +519,13 @@ class TestDXJobutilAddOutput(DXTestCase):
                                  ["analysisref", "thing"]] +
                                 [[classname,
                                   "'" +
-                                  json.dumps({"dnanexus_link": classname + "-" + self.dummy_hash}) +
+                                  json.dumps({"dnanexus_link": classname + "-" + DUMMY_HASH}) +
                                   "'"] for classname in self.data_obj_classes])
             for i, tc in enumerate(error_test_cases):
                 with self.assertSubprocessFailure(stderr_regexp='Value could not be parsed',
                                                   exit_code=3):
                     run(cmd_prefix + " ".join([str(i), "--class " + tc[0], tc[1]]))
 
-    def test_parallel_add(self):
-        temp_dir = tempfile.mkdtemp()
-        job_output_filename = os.path.join(temp_dir, 'job_output.json')
-
-        count = 200
-
-        shell_code = "\n".join("dx-jobutil-add-output myarray %d --class array:string --output %s &" %
-                               (i, pipes.quote(job_output_filename)) for i in range(count)) + \
-                     "\nwait"
-
-        run("/bin/bash -c \"" + shell_code + "\"", cwd=temp_dir)
-
-        with open(job_output_filename) as outputs:
-            output_spec = json.loads(outputs.read())
-            self.assertEquals(len(output_spec["myarray"]), count)
-            self.assertEquals(sorted(output_spec["myarray"]), sorted(str(i) for i in range(count)))
-
 
 class TestDXJobutilNewJob(DXTestCase):
     @classmethod
